@@ -21,14 +21,19 @@
 
 #include "collector.h"
 
-#include "fast_malloc.h"
-#include "internal.h"
-#include "list.h"
 #include "value.h"
+#include "internal.h"
 
 #if APPLE_CHANGES
 #include <CoreFoundation/CoreFoundation.h>
 #include <cxxabi.h>
+#endif
+
+#include <collector.h>
+#include <value.h>
+#include <internal.h>
+
+#if APPLE_CHANGES
 #include <pthread.h>
 #include <mach/mach_port.h>
 #include <mach/task.h>
@@ -58,7 +63,7 @@ struct CollectorCell {
     double memory[CELL_ARRAY_LENGTH];
     struct {
       void *zeroIfFree;
-      ptrdiff_t next;
+      CollectorCell *next;
     } freeCell;
   } u;
 };
@@ -81,7 +86,7 @@ struct CollectorHeap {
   int usedOversizeCells;
 
   int numLiveObjects;
-  int numLiveObjectsAtLastCollect;
+  int numAllocationsSinceLastCollect;
 };
 
 static CollectorHeap heap = {NULL, 0, 0, 0, NULL, 0, 0, 0, 0};
@@ -92,29 +97,29 @@ void* Collector::allocate(size_t s)
 {
   assert(Interpreter::lockCount() > 0);
 
+  if (s == 0)
+    return 0L;
+  
   // collect if needed
-  int numLiveObjects = heap.numLiveObjects;
-  if (numLiveObjects - heap.numLiveObjectsAtLastCollect >= ALLOCATIONS_PER_COLLECTION) {
+  if (++heap.numAllocationsSinceLastCollect >= ALLOCATIONS_PER_COLLECTION) {
     collect();
-    numLiveObjects = heap.numLiveObjects;
   }
   
-  if (s > static_cast<size_t>(CELL_SIZE)) {
+  if (s > (unsigned)CELL_SIZE) {
     // oversize allocator
     if (heap.usedOversizeCells == heap.numOversizeCells) {
       heap.numOversizeCells = MAX(MIN_ARRAY_SIZE, heap.numOversizeCells * GROWTH_FACTOR);
-      heap.oversizeCells = (CollectorCell **)kjs_fast_realloc(heap.oversizeCells, heap.numOversizeCells * sizeof(CollectorCell *));
+      heap.oversizeCells = (CollectorCell **)realloc(heap.oversizeCells, heap.numOversizeCells * sizeof(CollectorCell *));
     }
     
-    void *newCell = kjs_fast_malloc(s);
+    void *newCell = malloc(s);
     heap.oversizeCells[heap.usedOversizeCells] = (CollectorCell *)newCell;
     heap.usedOversizeCells++;
-    heap.numLiveObjects = numLiveObjects + 1;
+    heap.numLiveObjects++;
 
 #if !USE_CONSERVATIVE_GC
     ((ValueImp *)(newCell))->_flags = 0;
 #endif
-
     return newCell;
   }
   
@@ -137,10 +142,10 @@ void* Collector::allocate(size_t s)
     
     if (heap.usedBlocks == heap.numBlocks) {
       heap.numBlocks = MAX(MIN_ARRAY_SIZE, heap.numBlocks * GROWTH_FACTOR);
-      heap.blocks = (CollectorBlock **)kjs_fast_realloc(heap.blocks, heap.numBlocks * sizeof(CollectorBlock *));
+      heap.blocks = (CollectorBlock **)realloc(heap.blocks, heap.numBlocks * sizeof(CollectorBlock *));
     }
     
-    targetBlock = (CollectorBlock *)kjs_fast_calloc(1, sizeof(CollectorBlock));
+    targetBlock = (CollectorBlock *)calloc(1, sizeof(CollectorBlock));
     targetBlock->freeList = targetBlock->cells;
     heap.blocks[heap.usedBlocks] = targetBlock;
     heap.usedBlocks++;
@@ -148,19 +153,24 @@ void* Collector::allocate(size_t s)
   
   // find a free spot in the block and detach it from the free list
   CollectorCell *newCell = targetBlock->freeList;
-  
-  // "next" field is a byte offset -- 0 means next cell, so a zeroed block is already initialized
-  // could avoid the casts by using a cell offset, but this avoids a relatively-slow multiply
-  targetBlock->freeList = reinterpret_cast<CollectorCell *>(reinterpret_cast<char *>(newCell + 1) + newCell->u.freeCell.next);
+
+  if (newCell->u.freeCell.next != NULL) {
+    targetBlock->freeList = newCell->u.freeCell.next;
+  } else if (targetBlock->usedCells == (CELLS_PER_BLOCK - 1)) {
+    // last cell in this block
+    targetBlock->freeList = NULL;
+  } else {
+    // all next pointers are initially 0, meaning "next cell"
+    targetBlock->freeList = newCell + 1;
+  }
 
   targetBlock->usedCells++;
-  heap.numLiveObjects = numLiveObjects + 1;
+  heap.numLiveObjects++;
 
 #if !USE_CONSERVATIVE_GC
   ((ValueImp *)(newCell))->_flags = 0;
 #endif
-
-  return newCell;
+  return (void *)(newCell);
 }
 
 #if TEST_CONSERVATIVE_GC || USE_CONSERVATIVE_GC
@@ -329,10 +339,8 @@ void Collector::markStackObjectsConservatively()
 
 void Collector::markProtectedObjects()
 {
-  int size = ProtectedValues::_tableSize;
-  ProtectedValues::KeyValue *table = ProtectedValues::_table;
-  for (int i = 0; i < size; i++) {
-    ValueImp *val = table[i].key;
+  for (int i = 0; i < ProtectedValues::_tableSize; i++) {
+    ValueImp *val = ProtectedValues::_table[i].key;
     if (val && !val->marked()) {
       val->mark();
     }
@@ -364,7 +372,6 @@ bool Collector::collect()
 
   markStackObjectsConservatively();
   markProtectedObjects();
-  List::markProtectedLists();
 #endif
 
 #if TEST_CONSERVATIVE_GC
@@ -421,22 +428,20 @@ bool Collector::collect()
   // SWEEP: delete everything with a zero refcount (garbage) and unmark everything else
   
   int emptyBlocks = 0;
-  int numLiveObjects = heap.numLiveObjects;
 
   for (int block = 0; block < heap.usedBlocks; block++) {
     CollectorBlock *curBlock = heap.blocks[block];
 
     int minimumCellsToProcess = curBlock->usedCells;
 
-    for (int i = 0; i < CELLS_PER_BLOCK; i++) {
-      if (minimumCellsToProcess < i) {
+    for (int cell = 0; cell < CELLS_PER_BLOCK; cell++) {
+      if (minimumCellsToProcess < cell) {
 	goto skip_block_sweep;
       }
 
-      CollectorCell *cell = curBlock->cells + i;
-      ValueImp *imp = reinterpret_cast<ValueImp *>(cell);
+      ValueImp *imp = (ValueImp *)(curBlock->cells + cell);
 
-      if (cell->u.freeCell.zeroIfFree != 0) {
+      if (((CollectorCell *)imp)->u.freeCell.zeroIfFree != 0) {
 #if USE_CONSERVATIVE_GC
 	if (!imp->_marked)
 #else
@@ -447,13 +452,13 @@ bool Collector::collect()
 	  // emulate destructing part of 'operator delete()'
 	  imp->~ValueImp();
 	  curBlock->usedCells--;
-	  numLiveObjects--;
+	  heap.numLiveObjects--;
 	  deleted = true;
 
 	  // put it on the free list
-	  cell->u.freeCell.zeroIfFree = 0;
-	  cell->u.freeCell.next = reinterpret_cast<char *>(curBlock->freeList) - reinterpret_cast<char *>(cell + 1);
-	  curBlock->freeList = cell;
+	  ((CollectorCell *)imp)->u.freeCell.zeroIfFree = 0;
+	  ((CollectorCell *)imp)->u.freeCell.next = curBlock->freeList;
+	  curBlock->freeList = (CollectorCell *)imp;
 
 	} else {
 #if USE_CONSERVATIVE_GC
@@ -475,7 +480,7 @@ bool Collector::collect()
       emptyBlocks++;
       if (emptyBlocks > SPARE_EMPTY_BLOCKS) {
 #if !DEBUG_COLLECTOR
-	kjs_fast_free(heap.blocks[block]);
+	free(heap.blocks[block]);
 #endif
 	// swap with the last block so we compact as we go
 	heap.blocks[block] = heap.blocks[heap.usedBlocks - 1];
@@ -484,7 +489,7 @@ bool Collector::collect()
 
 	if (heap.numBlocks > MIN_ARRAY_SIZE && heap.usedBlocks < heap.numBlocks / LOW_WATER_FACTOR) {
 	  heap.numBlocks = heap.numBlocks / GROWTH_FACTOR; 
-	  heap.blocks = (CollectorBlock **)kjs_fast_realloc(heap.blocks, heap.numBlocks * sizeof(CollectorBlock *));
+	  heap.blocks = (CollectorBlock **)realloc(heap.blocks, heap.numBlocks * sizeof(CollectorBlock *));
 	}
       } 
     }
@@ -509,7 +514,7 @@ bool Collector::collect()
 #if DEBUG_COLLECTOR
       heap.oversizeCells[cell]->u.freeCell.zeroIfFree = 0;
 #else
-      kjs_fast_free((void *)imp);
+      free((void *)imp);
 #endif
 
       // swap with the last oversize cell so we compact as we go
@@ -517,11 +522,11 @@ bool Collector::collect()
 
       heap.usedOversizeCells--;
       deleted = true;
-      numLiveObjects--;
+      heap.numLiveObjects--;
 
       if (heap.numOversizeCells > MIN_ARRAY_SIZE && heap.usedOversizeCells < heap.numOversizeCells / LOW_WATER_FACTOR) {
 	heap.numOversizeCells = heap.numOversizeCells / GROWTH_FACTOR; 
-	heap.oversizeCells = (CollectorCell **)kjs_fast_realloc(heap.oversizeCells, heap.numOversizeCells * sizeof(CollectorCell *));
+	heap.oversizeCells = (CollectorCell **)realloc(heap.oversizeCells, heap.numOversizeCells * sizeof(CollectorCell *));
       }
 
     } else {
@@ -536,10 +541,9 @@ bool Collector::collect()
     }
   }
   
-  heap.numLiveObjects = numLiveObjects;
-  heap.numLiveObjectsAtLastCollect = numLiveObjects;
+  heap.numAllocationsSinceLastCollect = 0;
   
-  memoryFull = (numLiveObjects >= KJS_MEM_LIMIT);
+  memoryFull = (heap.numLiveObjects >= KJS_MEM_LIMIT);
 
   return deleted;
 }
@@ -603,10 +607,8 @@ int Collector::numReferencedObjects()
   int count = 0;
 
 #if USE_CONSERVATIVE_GC
-  int size = ProtectedValues::_tableSize;
-  ProtectedValues::KeyValue *table = ProtectedValues::_table;
-  for (int i = 0; i < size; i++) {
-    ValueImp *val = table[i].key;
+  for (int i = 0; i < ProtectedValues::_tableSize; i++) {
+    ValueImp *val = ProtectedValues::_table[i].key;
     if (val) {
       ++count;
     }
@@ -643,10 +645,8 @@ const void *Collector::rootObjectClasses()
   CFMutableSetRef classes = CFSetCreateMutable(NULL, 0, &kCFTypeSetCallBacks);
 
 #if USE_CONSERVATIVE_GC
-  int size = ProtectedValues::_tableSize;
-  ProtectedValues::KeyValue *table = ProtectedValues::_table;
-  for (int i = 0; i < size; i++) {
-    ValueImp *val = table[i].key;
+  for (int i = 0; i < ProtectedValues::_tableSize; i++) {
+    ValueImp *val = ProtectedValues::_table[i].key;
     if (val) {
       const char *mangled_name = typeid(*val).name();
       int status;
